//////////////////////////////////////////////////////////////////////////
//                                                                      //
// pciload.h -                                                          //
//                                                                      //
//      Load and exchange library function declaration.                 //
//      For dual-processor PCI board NM1.                               //
//                                                                      //
// Copyright (c) 1996-1998 RC Module Inc.                               //
//                                                                      //
//                                                                      //
// $Revision:: 13    $      $Date:: 25.03.99 13:23   $                  //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef _PCI_LOAD_H
#define _PCI_LOAD_H

#define PLOAD_VERSION "1.0"
#define PLOAD_MAJOR_VERSION 1
#define PLOAD_MINOR_VERSION 0

#ifdef _MAKE_DLL
#define DECLSPEC __declspec( dllexport )
#else
#define DECLSPEC __declspec( dllimport )
#endif

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

        // 32 bit unsigned. NM6403 memory element type.
    typedef unsigned long   PL_Word;

        // 32 bit unsigned. NM6403 address type.
    typedef unsigned long   PL_Addr;

        // Board descriptor
    typedef struct NM6403DualBoard PL_Board;

        // Processor on board descriptor.
    struct PL_Access;

        // Library functions return values.
        // All library functions return result code.
    enum RetValue {
        PL_OK       = 0,    // OK.
        PL_ERROR    = 1,    // Error.
        PL_TIMEOUT  = 2,    // Timeout of wait for operation.
        PL_FILE     = 3     // Can't find file for load.
    };


        //------------------
        // Common functions.
        //------------------
    
        // Return version of library in variable pointed by 'version'.
    DECLSPEC int PL_GetVersion( int * version );

        // Return number of detected boards in variable pointed by 'count'.
    DECLSPEC int PL_GetBoardCount( int * count );

        // Create descriptor for board number 'index'.
        // Return descriptor in variable pointed by 'board'.
        // Boards index count from 0.
    DECLSPEC int PL_GetBoardDesc( int index, PL_Board ** board  );

        // Close board descriptor. Free descriptor memory.
    DECLSPEC int PL_CloseBoardDesc( PL_Board * board );

        //-----------------
        // Board functions.
        //-----------------

        // Send RESET signal to board and all processors on board.
    DECLSPEC int PL_ResetBoard( PL_Board * board );

        // Load init code to board. Call this function after reset board
        // and before loading user program to processors.
    DECLSPEC int PL_LoadInitCode( PL_Board * board );

        // Load init code only to one processor on board.
        // Processors on board counts 1 and 2.
        // IMPORTANT: Don't use this function for initializing 
        // separately processors on board. It can issue board deadlock.
        // Instead use PL_LoadInitCode for initializing both processors.
    DECLSPEC int PL_LoadInitCode_Single( PL_Board * board, int procNo );

        // Create descriptor for processor number 'procNo' on board.
        // Return processor descriptor in variable pointed by 'access'.
        // Processors on board counts 1 and 2.
    DECLSPEC int PL_GetAccess ( PL_Board * board, int procNo, PL_Access ** access  );

        // Close processor descriptor.
    DECLSPEC int PL_CloseAccess( PL_Access * access );

        // On-board light enable/disable function.
        // 1 - light on.
        // 0 - light off.
    DECLSPEC int PL_Light( PL_Board *board, int Enable );

        //---------------------
        // Processor functions.
        //---------------------

        // Load user program on processor and start execution.
    DECLSPEC int PL_LoadProgramFile( PL_Access * access, const char * filename );

        // Load user program on processor and start execution.
        // Special version for for single processor.
        // IMPORTANT: use this function only with PL_LoadInitCode_Single
        // init function and only to initialized processor.
    DECLSPEC int PL_LoadProgramFile_Single( PL_Access * access, const char * filename );


        // Write array in shared memory.
        // block    - Pointer to source array in PC memory.
        // len      - Size of array in 32-bit words.
        // address  - Address of destination array in NM6403 memory.
    DECLSPEC int PL_WriteMemBlock( PL_Access * access, PL_Word * block, PL_Addr address, PL_Word len );

        // Read array from shared memory.
        // block    - Pointer to dest buffer in PC memory.
        // len      - Size of array in 32-bit words.
        // address  - Address of source array in NM6403 memory.
    DECLSPEC int PL_ReadMemBlock( PL_Access * access, PL_Word * block, PL_Addr address, PL_Word len );

        // Barrier syncronization with program on board processor.
        // value        - value sent to processor.
        // returnValue  - value received from processor.
    DECLSPEC int PL_Sync( PL_Access * access, int value, int * returnValue );


        // Barrier syncronization with program on board processor.
        // Additionally perform two values exchange.
        // value       - value sent to processor.
        // outAddress  - address sent to processor.
        // outLen      - size sent to processor.
        // returnValue - value received from processor.
        // inAddress   - address received from processor.
        // inLen       - size received from processor.
        // Values return only if pointers are not NULL.
    DECLSPEC int PL_SyncArray( 
            PL_Access * access,     // Processor descriptor.

            int value,              // Value sent to processor.
            PL_Addr outAddress,     // Address sent to processor.
            PL_Word outLen,         // Size sent to processor.

            int * returnValue,      // Value received from processor.
            PL_Addr * inAddress,    // Address received from processor.
            PL_Word *inLen          // Size received from processor.
        );

        // Send interrupt on processor.
    DECLSPEC int PL_Interrupt( PL_Access * access );

        // Return status bits in variable 'status'.
    DECLSPEC int PL_GetStatus( PL_Access * access, PL_Word * status );

        // Set timeout in milliseconds for waiting function.
        // Used in all PL_SyncXXX functions.
    DECLSPEC int PL_SetTimeout( int Timeout );

#ifdef __cplusplus
}
#endif


#endif // _PCI_LOAD_H
